@@ -29,21 +29,19 @@ extern int tests_failed;
 /**
  * Basic test assertion with reporting
  */
-#define TEST_ASSERT(condition, message)                          \
-    do                                                           \
-    {                                                            \
-        if (condition)                                           \
-        {                                                        \
-            tests_passed++;                                      \
-            printf("✓ %s\n", message);                           \
-        }                                                        \
-        else                                                     \
-        {                                                        \
-            tests_failed++;                                      \
-            printf("✗ %s\n", message);                           \
-            /* Print file and line where the assertion failed */ \
-            printf("  at %s:%d\n", __FILE__, __LINE__);          \
-        }                                                        \
+#define TEST_ASSERT(condition, message)      \
+    do                                       \
+    {                                        \
+        if (condition)                       \
+        {                                    \
+            printf("✓ PASS: %s\n", message); \
+            tests_passed++;                  \
+        }                                    \
+        else                                 \
+        {                                    \
+            printf("✗ FAIL: %s\n", message); \
+            tests_failed++;                  \
+        }                                    \
     } while (0)
 
 /**
@@ -64,7 +62,6 @@ static regex_t create_regex(const char *pattern, bool case_sensitive)
         char error_buf[256];
         regerror(rc, &regex_obj, error_buf, sizeof(error_buf));
         fprintf(stderr, "Regex compilation error: %s for pattern '%s'\n", error_buf, pattern);
-        // In a real test suite, might use assert(rc == 0) or handle differently
         exit(1); // Exit if compilation fails, as tests depend on it
     }
 
@@ -114,17 +111,13 @@ void test_complex_regex(void)
     size_t haystack_len = strlen(haystack);
 
     // Word boundaries and case-insensitive
-    // Note: POSIX ERE might not support \b reliably everywhere.
-    // Testing simple "hello" case-insensitively.
     regex_t re1 = create_regex("hello", false); // Case-insensitive
     uint64_t matches_hello = regex_search_compat(haystack, haystack_len, &re1, SIZE_MAX);
-    // Expecting 3 matches: "Hello", "hello", "HELLO"
     TEST_ASSERT(matches_hello == 3, "Case-insensitive regex finds 'hello' three times");
 
     // Digit sequences
     regex_t re2 = create_regex("[0-9]+", true); // Match one or more digits
     uint64_t matches_digits = regex_search_compat(haystack, haystack_len, &re2, SIZE_MAX);
-    // Expecting 3 matches: "123", "456", "789"
     TEST_ASSERT(matches_digits == 3, "Regex finds three digit sequences");
 
     // More complex pattern with alternation and groups
@@ -132,7 +125,6 @@ void test_complex_regex(void)
     size_t text_len = strlen(text);
     regex_t re3 = create_regex("(apple|orange)", true); // Match "apple" or "orange"
     uint64_t matches_fruits = regex_search_compat(text, text_len, &re3, SIZE_MAX);
-    // Expecting 4 matches: "apple", "orange", "apple", "orange"
     TEST_ASSERT(matches_fruits == 4, "Regex finds 'apple' or 'orange' four times");
 
     // Free compiled regex objects
@@ -148,36 +140,24 @@ void test_regex_multiple_matches(void)
 {
     printf("\n=== Regex Multiple Matches Tests ===\n");
 
-    // Test with repeating patterns
     const char *text1 = "aaa bbb aaa bbb aaa";
     size_t text1_len = strlen(text1);
     regex_t re1 = create_regex("aaa", true);
     uint64_t matches_aaa = regex_search_compat(text1, text1_len, &re1, SIZE_MAX);
-    // Expecting 3 matches: "aaa", "aaa", "aaa"
     TEST_ASSERT(matches_aaa == 3, "Regex finds 'aaa' three times");
 
-    // Test with overlapping potential matches (regex should handle non-overlapping)
-    // POSIX regexec finds non-overlapping matches.
-    const char *text2 = "abababababa"; // Length 11
+    const char *text2 = "abababababa";
     size_t text2_len = strlen(text2);
     regex_t re2 = create_regex("ababa", true);
     uint64_t matches_ababa = regex_search_compat(text2, text2_len, &re2, SIZE_MAX);
-    // Expecting 2 non-overlapping matches: "ababa" at index 0, "ababa" at index 4 (or maybe 5 depending on engine?)
-    // Let's trace: Match 1: 0-5 ("ababa"). Next search starts after match.
-    // Search "bababa": Match 2: index 5-10 ("ababa"). Next search starts after.
-    // Search "ba": No match. Expected: 2
     TEST_ASSERT(matches_ababa == 2, "Regex finds 'ababa' twice (non-overlapping)");
 
-    // Test with line-based matches (using ^ and $)
     const char *text3 = "Line 1: apple\nLine 2: orange\nLine 3: apple\nLine 4: banana";
     size_t text3_len = strlen(text3);
-    // Match lines starting with "Line <digit>:" followed by "apple" or "orange" until the end of the line ($)
     regex_t re3 = create_regex("^Line [0-9]+: (apple|orange)$", true);
     uint64_t matches_lines = regex_search_compat(text3, text3_len, &re3, SIZE_MAX);
-    // Expecting 3 matches: Line 1, Line 2, Line 3
     TEST_ASSERT(matches_lines == 3, "Regex finds three lines with 'apple' or 'orange'");
 
-    // Free compiled regex objects
     regfree(&re1);
     regfree(&re2);
     regfree(&re3);
@@ -190,38 +170,29 @@ void test_regex_edge_cases(void)
 {
     printf("\n=== Regex Edge Cases Tests ===\n");
 
-    // Test empty string
     const char *empty = "";
     size_t empty_len = 0;
 
-    // Match any character in empty string (should fail)
-    regex_t re1 = create_regex(".", true); // '.' matches any single character except newline by default
+    regex_t re1 = create_regex(".", true);
     uint64_t matches_dot = regex_search_compat(empty, empty_len, &re1, SIZE_MAX);
     TEST_ASSERT(matches_dot == 0, "Regex '.' doesn't match in empty string");
 
-    // Match empty pattern in empty string (should succeed once)
-    // '^' matches start, '$' matches end. In an empty string, start and end are the same.
     regex_t re2 = create_regex("^$", true);
     uint64_t matches_empty = regex_search_compat(empty, empty_len, &re2, SIZE_MAX);
     TEST_ASSERT(matches_empty == 1, "Regex '^$' matches empty string once");
 
-    // Test with pattern longer than text
     const char *short_text = "abc";
     size_t short_len = strlen(short_text);
     regex_t re3 = create_regex("abcdef", true);
     uint64_t matches_long = regex_search_compat(short_text, short_len, &re3, SIZE_MAX);
     TEST_ASSERT(matches_long == 0, "Regex longer than text doesn't match");
 
-    // Test with complex pattern that might cause excessive backtracking in some engines
-    // POSIX regex engines are generally required to be non-exponential, but complex patterns can still be slow.
-    const char *complex_text = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab"; // 30 'a's then 'b'
+    const char *complex_text = "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaab";
     size_t complex_len = strlen(complex_text);
-    // This pattern matches zero or more 'a's many times, followed by 'b'. Should match once.
     regex_t re4 = create_regex("a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*a*b", true);
     uint64_t matches_backtrack = regex_search_compat(complex_text, complex_len, &re4, SIZE_MAX);
     TEST_ASSERT(matches_backtrack == 1, "Complex regex with backtracking matches correctly");
 
-    // Free compiled regex objects
     regfree(&re1);
     regfree(&re2);
     regfree(&re3);
@@ -235,35 +206,28 @@ void test_regex_overlapping(void)
 {
     printf("\n=== Regex Overlapping Tests ===\n");
 
-    // Simple overlapping case: "aa" in "aaaa"
     const char *text1 = "aaaa";
     size_t text1_len = strlen(text1);
     regex_t re1 = create_regex("aa", true);
     uint64_t matches_aa = regex_search_compat(text1, text1_len, &re1, SIZE_MAX);
-    // Expect 2 non-overlapping matches: "aa" at 0, "aa" at 2.
     TEST_ASSERT(matches_aa == 2, "Regex finds 'aa' twice (non-overlapping)");
 
-    // Complex overlapping case: "aba" in "ababababa"
-    const char *text2 = "ababababa"; // Length 9
+    const char *text2 = "ababababa";
     size_t text2_len = strlen(text2);
     regex_t re2 = create_regex("aba", true);
     uint64_t matches_aba = regex_search_compat(text2, text2_len, &re2, SIZE_MAX);
-    // Update expectation to match actual implementation behavior
     TEST_ASSERT(matches_aba == 2, "Regex finds 2 non-overlapping 'aba' patterns");
     printf("  Found %" PRIu64 " instances of 'aba' in '%s'\n", matches_aba, text2);
 
-    // Zero-width assertions shouldn't cause infinite loops
     const char *text3 = "abcdef";
     size_t text3_len = strlen(text3);
-    regex_t re3 = create_regex("^|$", true); // Match beginning or end of the string/line
+    regex_t re3 = create_regex("^|$", true);
     uint64_t matches_bounds = regex_search_compat(text3, text3_len, &re3, SIZE_MAX);
 
-    // Make platform-agnostic: accept either 1 or 2 matches since behavior differs between regex engines
     bool valid_count = (matches_bounds == 1 || matches_bounds == 2);
     TEST_ASSERT(valid_count, "Regex finds correct zero-width assertions count");
     printf("  Found %" PRIu64 " zero-width assertions\n", matches_bounds);
 
-    // Free compiled regex objects
     regfree(&re1);
     regfree(&re2);
     regfree(&re3);
@@ -276,16 +240,13 @@ void test_regex_report_limit(void)
 {
     printf("\n=== Regex Max Count Limit Tests ===\n");
 
-    // Create a text with multiple matches
-    const char *text = "aaa bbb aaa ccc aaa ddd aaa"; // 4 "aaa" matches
+    const char *text = "aaa bbb aaa ccc aaa ddd aaa";
     size_t text_len = strlen(text);
-    match_result_t *result = NULL; // Not needed for count checks
+    match_result_t *result = NULL;
 
-    // Use helper to create params, then modify max_count
     search_params_t params = create_regex_params("aaa", true, false, false);
 
-    // Test with different limits
-    params.max_count = SIZE_MAX; // Effectively no limit
+    params.max_count = SIZE_MAX;
     uint64_t matches_full = regex_search(&params, text, text_len, result);
     TEST_ASSERT(matches_full == 4, "Regex finds all 4 'aaa' with no limit");
 
@@ -305,7 +266,6 @@ void test_regex_report_limit(void)
     uint64_t matches_limit0 = regex_search(&params, text, text_len, result);
     TEST_ASSERT(matches_limit0 == 0, "Regex finds 0 with limit 0");
 
-    // Test with tracking positions
     params.track_positions = true;
     params.max_count = 2;
     result = match_result_init(5);
@@ -315,7 +275,6 @@ void test_regex_report_limit(void)
     match_result_free(result);
     result = NULL;
 
-    // Cleanup compiled regex allocated by helper
     cleanup_params(&params);
 }
 
@@ -326,21 +285,19 @@ void test_regex_vs_literal_performance(void)
 {
     printf("\n=== Regex vs. Literal Performance Tests ===\n");
 
-    // Create a large text with occasional matches
-    const size_t size = 100 * 1024; // 100KB for quick test
+    const size_t size = 100 * 1024;
     char *large_text = (char *)malloc(size + 1);
     if (!large_text)
     {
-        perror("Cannot allocate memory for performance test");
+        perror("Failed to allocate memory for large_text in performance test");
+        tests_failed++;
         return;
     }
 
-    // Fill with 'a's and occasional 'b's (every 1000th char)
     size_t expected_literal_count = 0;
     for (size_t i = 0; i < size; i++)
     {
-        // Ensure 'b' is placed correctly
-        if (i > 0 && i % 1000 == 0)
+        if ((i + 1) % 1000 == 0)
         {
             large_text[i] = 'b';
             expected_literal_count++;
@@ -350,102 +307,81 @@ void test_regex_vs_literal_performance(void)
             large_text[i] = 'a';
         }
     }
-    // Handle potential edge case if size is multiple of 1000
-    if (size > 0 && size % 1000 == 0)
-    {
-        large_text[size - 1] = 'a'; // Avoid ending with 'b' if it complicates things
-    }
-    // Place one 'b' at the start if size > 0
+
     if (size > 0)
     {
+        if (large_text[0] != 'b')
+        {
+            expected_literal_count++;
+        }
         large_text[0] = 'b';
-        expected_literal_count++;
     }
 
     large_text[size] = '\0';
 
-    // Create search pattern
     const char *pattern = "b";
     size_t pattern_len = 1;
 
     printf("Comparing performance for %zu KB text with single-char pattern '%s'...\n", size / 1024, pattern);
-    printf("Expected matches: %zu\n", expected_literal_count); // Debug print
+    printf("Expected matches: %zu\n", expected_literal_count);
 
-    // --- Measure literal search time ---
     clock_t start_lit = clock();
 
-    // Create search params for Boyer-Moore, configured for match counting
-    search_params_t bm_params = {
-        .pattern = pattern,
-        .pattern_len = pattern_len,
-        .case_sensitive = true,
-        .use_regex = false,
-        .track_positions = false,   // Don't track positions
-        .count_lines_mode = false,  // Don't count lines
-        .count_matches_mode = true, // Indicate intent to count matches (though BM uses !track && !lines)
-        .compiled_regex = NULL,
-        .max_count = SIZE_MAX,
-        // Assign multi-pattern fields for consistency if helper not used
-        .patterns = (const char **)&pattern, // Point to the single pattern
-        .pattern_lens = &pattern_len,
-        .num_patterns = 1};
+    search_params_t bm_params = {0};
+    bm_params.patterns = malloc(sizeof(char *));
+    bm_params.pattern_lens = malloc(sizeof(size_t));
+
+    if (!bm_params.patterns || !bm_params.pattern_lens)
+    {
+        perror("Failed to allocate memory for bm_params fields in performance test");
+        if (bm_params.patterns)
+            free((void *)bm_params.patterns);
+        if (bm_params.pattern_lens)
+            free(bm_params.pattern_lens);
+        free(large_text);
+        tests_failed++;
+        return;
+    }
+    bm_params.patterns[0] = pattern;
+    bm_params.pattern_lens[0] = pattern_len;
+    bm_params.num_patterns = 1;
+
+    bm_params.pattern = bm_params.patterns[0];
+    bm_params.pattern_len = bm_params.pattern_lens[0];
+
+    bm_params.case_sensitive = true;
+    bm_params.use_regex = false;
+    bm_params.count_lines_mode = false;
+    bm_params.count_matches_mode = true;
+    bm_params.track_positions = false;
+    bm_params.whole_word = false;
+    bm_params.compiled_regex = NULL;
+    bm_params.ac_trie = NULL;
+    bm_params.max_count = SIZE_MAX;
 
-    // Use the actual boyer_moore_search function
     uint64_t literal_count = boyer_moore_search(&bm_params, large_text, size, NULL);
 
     clock_t end_lit = clock();
     double literal_time = ((double)(end_lit - start_lit)) / CLOCKS_PER_SEC;
+    printf("  Literal search (BM): %f seconds (found %" PRIu64 " matches)\n", literal_time, literal_count);
     TEST_ASSERT(literal_count == expected_literal_count, "Literal search found correct count");
+    cleanup_params(&bm_params);
 
-    // --- Measure regex search time ---
-    regex_t re = create_regex(pattern, true); // Case-sensitive regex "b"
+    regex_t re = create_regex(pattern, true);
     clock_t start_re = clock();
-    // Create params for regex search, configured for match counting
-    search_params_t re_params = {
-        .pattern = pattern, // Informational
-        .pattern_len = pattern_len,
-        .case_sensitive = true, // Set via regcomp flags
-        .use_regex = true,
-        .track_positions = false,   // Don't track positions
-        .count_lines_mode = false,  // Don't count lines
-        .count_matches_mode = true, // Indicate intent
-        .compiled_regex = &re,
-        .max_count = SIZE_MAX,
-        // Assign multi-pattern fields
-        .patterns = (const char **)&pattern,
-        .pattern_lens = &pattern_len,
-        .num_patterns = 1};
-    // Use the actual regex_search function
-    uint64_t regex_count = regex_search(&re_params, large_text, size, NULL);
+    uint64_t regex_matches = regex_search_compat(large_text, size, &re, SIZE_MAX);
     clock_t end_re = clock();
     double regex_time = ((double)(end_re - start_re)) / CLOCKS_PER_SEC;
-    TEST_ASSERT(regex_count == expected_literal_count, "Regex search found correct count");
+    printf("  Regex search: %f seconds (found %" PRIu64 " matches)\n", regex_time, regex_matches);
+    TEST_ASSERT(regex_matches == expected_literal_count, "Regex search found correct count");
 
-    // --- Report times ---
-    printf("  Literal search: %" PRIu64 " matches in %.6f seconds\n", literal_count, literal_time);
-    printf("  Regex search:   %" PRIu64 " matches in %.6f seconds\n", regex_count, regex_time);
-    // Avoid division by zero if literal_time is extremely small
-    if (literal_time > 1e-9 && regex_time >= 0) // Ensure times are non-negative
-    {
-        printf("  Regex is %.2f times slower than literal search for this pattern\n",
-               regex_time / literal_time);
-    }
-    else if (regex_time < 1e-9 && literal_time < 1e-9)
-    {
-        printf("  Both searches too fast to calculate ratio.\n");
-    }
-    else if (literal_time < 1e-9)
-    {
-        printf("  Literal search too fast to calculate ratio.\n");
-    }
-    else
-    {
-        printf("  Could not calculate ratio (regex_time=%.6f, literal_time=%.6f).\n", regex_time, literal_time);
-    }
-
-    // Cleanup
     regfree(&re);
     free(large_text);
+
+    if (literal_time > 1e-9 && regex_time > 1e-9)
+    {
+        printf("  Literal search was %.2fx faster than regex search.\n", regex_time / literal_time);
+    }
 }
 
 /**
@@ -458,24 +394,20 @@ void test_regex_line_extraction(void)
     const char *text = "example: first match\nother line\nexample: second match";
     size_t text_len = strlen(text);
 
-    // Create the search parameters structure
     search_params_t params = {
-        .pattern = "example:.*",             // Regex pattern to match lines starting with "example:"
-        .pattern_len = strlen("example:.*"), // Length is informational for regex
+        .pattern = "example:.*",
+        .pattern_len = strlen("example:.*"),
         .case_sensitive = true,
         .use_regex = true,
-        .track_positions = true,     // Enable position tracking
-        .count_lines_mode = false,   // Not counting lines
-        .count_matches_mode = false, // Not just counting matches
-        .compiled_regex = NULL,      // Will be set after compilation
-        .max_count = SIZE_MAX        // No limit for this test
-    };
-    // Assign multi-pattern fields
+        .track_positions = true,
+        .count_lines_mode = false,
+        .count_matches_mode = false,
+        .compiled_regex = NULL,
+        .max_count = SIZE_MAX};
     params.patterns = (const char **)&params.pattern;
     params.pattern_lens = &params.pattern_len;
     params.num_patterns = 1;
 
-    // Compile the regex
     regex_t regex_obj;
     int flags = REG_EXTENDED | REG_NEWLINE;
     int rc = regcomp(&regex_obj, params.pattern, flags);
@@ -484,43 +416,34 @@ void test_regex_line_extraction(void)
         char errbuf[256];
         regerror(rc, &regex_obj, errbuf, sizeof(errbuf));
         fprintf(stderr, "Failed to compile regex in line extraction test: %s\n", errbuf);
-        tests_failed++; // Mark test as failed
+        tests_failed++;
         return;
     }
-    params.compiled_regex = &regex_obj; // Assign compiled regex to params
+    params.compiled_regex = &regex_obj;
 
-    // Create a match_result_t structure to store the results
-    match_result_t *result = match_result_init(10); // Initial capacity of 10
+    match_result_t *result = match_result_init(10);
     if (!result)
     {
         fprintf(stderr, "Failed to create match_result in line extraction test\n");
         regfree(&regex_obj);
-        tests_failed++; // Mark test as failed
+        tests_failed++;
         return;
     }
 
-    // --- Call the original regex_search function ---
-    // Perform search using the original function signature, passing the result struct
     uint64_t match_count = regex_search(&params, text, text_len, result);
-    // --- End of original function call ---
 
-    // --- Verify results ---
     TEST_ASSERT(match_count == 2, "Regex found 2 matches");
     TEST_ASSERT(result->count == 2, "Result contains 2 positions");
 
-    // Check the specific offsets if we found 2 matches
     if (result->count == 2)
     {
-        // Match 1: "example: first match"
         TEST_ASSERT(result->positions[0].start_offset == 0, "First match starts at offset 0");
         TEST_ASSERT(result->positions[0].end_offset == 20, "First match ends at offset 20");
 
-        // Match 2: "example: second match"
         TEST_ASSERT(result->positions[1].start_offset == 32, "Second match starts at offset 32");
         TEST_ASSERT(result->positions[1].end_offset == 53, "Second match ends at offset 53");
     }
 
-    // Cleanup
     match_result_free(result);
     regfree(&regex_obj);
 }
@@ -539,7 +462,7 @@ void run_regex_tests(void)
     test_regex_overlapping();
     test_regex_report_limit();
     test_regex_vs_literal_performance();
-    test_regex_line_extraction(); // The test containing failures
+    test_regex_line_extraction();
 
     printf("\n--- Completed Regex Tests ---\n");
 }
